﻿-- ----------------------------------------------------------------------- --
--     Moncaí's Compact Experience Bar
--     Copyright (c) 2006-2011 Moncaí
--
--     May be copied and distributed freely as long as you do not
--     change this file.
--     
-- ----------------------------------------------------------------------- --

-- Default state
CEBVisible = true;
CEBLocked = false;

-- Constants
local LibMonoFont = "Interface\\AddOns\\MoncaiCEB\\art\\AuSans.otf";
local ThousandSeparator = ","
local TipTitle = "Moncai's Compact XP Bar"

if (GetLocale() == "deDE") then 			-- // -- deDE
	ThousandSeparator = "."
end

local function FmtBigNum(n)
        local left, num, right = string.match(n .. "", '^([^%d]*%d)(%d*)(.-)')
        return left..(num:reverse():gsub('(%d%d%d)', '%1'.. ","):reverse())..right
end

local CompactExperienceBar = CreateFrame("StatusBar","CompactExperienceBar",UIParent)
CompactExperienceBar:SetWidth(95);
CompactExperienceBar:SetHeight(50);
CompactExperienceBar:SetPoint("CENTER", UIParent, "CENTER", 0, 0);
CompactExperienceBar:SetFrameStrata("MEDIUM");	
CompactExperienceBar:SetClampedToScreen(1);
CompactExperienceBar:SetBackdrop({bgFile="Interface\\Tooltips\\UI-Tooltip-Background", edgeFile="", tile=1, tileSize=10, edgeSize=10, insets={left=2, right=2, top=2, bottom=2}});
CompactExperienceBar:SetBackdropColor(0,0,0,1)
CompactExperienceBar:SetMovable(1);
CompactExperienceBar:EnableMouse(1);

CompactExperienceBar:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar")
CompactExperienceBar:SetStatusBarColor(0,0,1)

local CEBarText = CompactExperienceBar:CreateFontString("CEBarText","OVERLAY");
CEBarText:SetFontObject(GameFontNormal);
CEBarText:SetPoint("CENTER", CompactExperienceBar, "CENTER", 0, 0);
CEBarText:SetJustifyH("CENTER");
CEBarText:SetJustifyV("CENTER");

local CompactExperienceBar1 = CreateFrame("Frame","CompactExperienceBar1",CompactExperienceBar)
CompactExperienceBar1:SetPoint("TOPLEFT", CompactExperienceBar, "TOPLEFT", -2, 1);
CompactExperienceBar1:SetPoint("BOTTOMRIGHT", CompactExperienceBar, "BOTTOMRIGHT", 2, -1);
CompactExperienceBar1:SetBackdrop({bgFile="", edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", tile=1, tileSize=10, edgeSize=10, insets={left=2, right=2, top=2, bottom=2}});
CompactExperienceBar1:SetBackdropColor(0,0,0,1)

CompactExperienceBar:RegisterEvent("PLAYER_ENTERING_WORLD");
CompactExperienceBar:RegisterEvent("VARIABLES_LOADED");
CompactExperienceBar:RegisterEvent("PLAYER_XP_UPDATE");
CompactExperienceBar:RegisterEvent("PLAYER_LOGIN");
CompactExperienceBar:RegisterEvent("PLAYER_LEVEL_UP");	

-- // Tooltip // --------------------------------------------

local CompactExperienceBarTooltip = CreateFrame( "GameTooltip", "CompactExperienceBarTooltip", nil, "GameTooltipTemplate" ); -- Tooltip name cannot be nil
CompactExperienceBarTooltip:SetOwner(CompactExperienceBar, "ANCHOR_NONE" );

CompactExperienceBar:SetScript("OnEnter", function(self, ...)
	local tip = CompactExperienceBarTooltip
	local r,g,b = _G.NORMAL_FONT_COLOR[1], _G.NORMAL_FONT_COLOR[2], _G.NORMAL_FONT_COLOR[3]
	local curxp, levelmax, rest = UnitXP("player") or 0, UnitXPMax("player") or 0, GetXPExhaustion() or 0;
	local togo = levelmax - curxp
	local perc = string.format("%.1f%%", (100*(curxp/levelmax)))
	local perbox = levelmax / 20
	
	local strcur   = string.format("%s %4.1fb %5.1f%%",  FmtBigNum(curxp), curxp / perbox, curxp/levelmax*100)
	local strtogo = string.format("%s %4.1fb %5.1f%%",  FmtBigNum(togo), togo / perbox, togo/levelmax*100)
	local strrest  = string.format("%s %4.1fb %5.1f%%",  FmtBigNum(rest), rest / perbox, rest/levelmax*100)
	
	tip:SetOwner(self, "ANCHOR_TOPRIGHT")
	tip:ClearLines();
	tip:AddLine(TipTitle, 0.1, 0.7, 0.1)
	tip:AddDoubleLine("XP", strcur, r,g,b, 0.5,0,0.5)
	tip:AddDoubleLine("To go", strtogo, r,g,b, 0.5,0,0.5)	
	tip:AddDoubleLine("Rested", strrest, r,g,b, 0,0,1)
	
	for i = 2, 4, 1 do
		_G["CompactExperienceBarTooltipTextRight" .. i]:SetFont(LibMonoFont, 12);
	end	
	tip:Show();
end)

CompactExperienceBar:SetScript("OnLeave", function(self, ...)
	CompactExperienceBarTooltip:Hide();
end)

-- // Event Handling // --------------------------------------------

CompactExperienceBar:SetScript("OnEvent", function(self, event, ...)
	local curxp = UnitXP("player")
	local levelmax = UnitXPMax("player")
	local perc = string.format("%.1f%%", (100*(curxp/levelmax)))
	local perbox = levelmax / 20

	local mystr = string.format("%.1fb - %.1fb tnl | %s",  curxp / perbox, (levelmax - curxp) / perbox, perc)

	local r=GetXPExhaustion()
	if -1 == (r or -1) then -- no rested
		CompactExperienceBar:SetStatusBarColor(0.5,0,0.5)
	else -- rested
		CompactExperienceBar:SetStatusBarColor(0,0,1)
		--mystr = mystr .. string.format(" %.1fr",  r / perbox)
	end
	
	CEBarText:SetText(mystr)
	CompactExperienceBar:SetWidth(CEBarText:GetStringWidth()+10)
	CompactExperienceBar:SetHeight(CEBarText:GetStringHeight()+10)
	
	CompactExperienceBar:SetMinMaxValues(0, levelmax)
	CompactExperienceBar:SetValue(curxp)

	if CEBVisible then 
		CompactExperienceBar:Show() 
	else
		CompactExperienceBar:Hide()
	end
end)

-- // Make frame movable // --------------------------------------------

CompactExperienceBar:SetScript("OnMouseDown", function() 
	if not CEBLocked then 
		CompactExperienceBar:StartMoving() 
	else 
		CompactExperienceBar:StopMovingOrSizing() 
	end 
end)

CompactExperienceBar:SetScript("OnMouseUp", function() CompactExperienceBar:StopMovingOrSizing() end)	

-- // Slash command // --------------------------------------------

SLASH_CEB1 = '/ceb';
local function slashcmdhandler(param)
	if param == "hide" then
		if CEBVisible then
			CEBVisible=false
			CompactExperienceBar:Hide() 
			print("Experience hidden, type /ceb show to show.")
		end
	elseif param=="show" then
		if not CEBVisible then
			CEBVisible=true
			CompactExperienceBar:Show()
			print("Experience displayed, type /ceb hide to hide.")
		end
	elseif param=="rested" then
		if GetXPExhaustion() == nil then
			print("You have no rested EXP.")
		else
			print("You have "..GetXPExhaustion().." rested EXP remaining.")
		end
	else
		print("No command or incorrect command entered. Valid commands are:");
		print("/ceb show (Show the compact experience bar.)");
		print("/ceb hide (Hide the compact experience bar.)");
		print("/ceb rested (Shows the amount of rested experience.)");
		print("Right click the compact experience bar to lock and unlock movement.");
	end
end
SlashCmdList["CEB"] = slashcmdhandler;

-- // Quest Log functionality; alternative display // --------------------------------------------

function calcboxes()
	local f = QuestInfoXPFramePoints

	local k, v , y = string.gmatch(f:GetText() or "", "(%d+),?(%d*),?(%d*)")();
	k = (k or "") .. (v or "") .. (y or "");	
	local xp = 0 + (tonumber( k ) or 0)	
	
	if xp > 0 then
		local levelmax=UnitXPMax("player")
		local perc = (xp / levelmax) * 20
		local amt = string.format("%.1f", perc)
		if amt == "0.0" then
			amt = string.format("%.2f", perc)
			if amt == "0.00" then
				amt = "0"
			end
		end
		local curxp=UnitXP("player")
		
		local result = string.format("%s +%sb %.1f%% %dx", FmtBigNum(xp), amt, (xp / levelmax) * 100, ((levelmax-curxp) / xp) + 0.5 )
		--Print("Quest has: " .. result)
		f:SetText(result)
	end
end

function XPHooker()
	hooksecurefunc("QuestInfo_ShowRewards", calcboxes)
	hooksecurefunc("QuestLog_UpdateQuestDetails", calcboxes)
	hooksecurefunc("QuestFrameRewardPanel_OnShow", calcboxes)
	hooksecurefunc("QuestFrameDetailPanel_OnShow", calcboxes)
	hooksecurefunc("QuestInfo_Display", calcboxes)
end

XPHooker()

if UnitLevel("player") >= 90 then
	CompactExperienceBar:Hide() print("Experience bar hidden, type /ceb show to show.")
	CEBVisible = false
end